@@ -1,4 +1,3 @@
-
 /*
  * bltGrAxis.c --
  *
@@ -25,6 +24,7 @@
  *
  */
 
+#include <assert.h>
 #include "blt.h"
 #include "bltGraph.h"
 #include "bltGrElem.h"
@@ -48,12 +48,18 @@
 #define UCEIL(x,u)	(ceil((x)/(u))*(u))
 #define UFLOOR(x,u)	(floor((x)/(u))*(u))
 
-#define MAJOR_TICK 	0.020	/* Length of a major tick */
-#define MINOR_TICK 	0.015	/* Length of a minor (sub)tick */
-#define LABEL_TICK 	0.030	/* Distance from graph to start of label */
-#define NUMDIGITS	12	/* Specifies the number of digits of
-				* accuracy used when outputting axis
-				* tick labels. */
+#define DEF_MAJOR_TICK 	0.030	/* Length of a major tick */
+#define DEF_MINOR_TICK 	0.015	/* Length of a minor (sub)tick */
+#define DEF_LABEL_TICK 	0.040	/* Distance from graph to start of label */
+
+#define NUMDIGITS	15	/* Specifies the number of digits of
+				 * accuracy used when outputting axis
+				 * tick labels. */
+enum PositionIndices {
+    MAJOR_TICK, MINOR_TICK, TICK_LABEL, AXIS_LINE
+};
+
+#define AVG_TICK_NUM_CHARS	16	/* Assumed average tick label size */
 
 enum LimitIndices {
     LMIN, LMAX
@@ -98,12 +104,9 @@ static Tk_CustomOption MaxLimitOption =
  *
  * Label --
  *
- * 	Structure contains the formatted string and either the x or
- *	y window coordinate position of the tick label (its center).
- *	There is a limit on number of characters in a tick label of
- *	32 which seems to be more than enough.  The notion here was
- * 	to be able to quickly allocate and deallocate space for the
- *	labels.
+ * 	Structure containing the formatted label and the screen
+ * 	coordinates position of the tick label (anchored at its
+ * 	center).
  *
  * ----------------------------------------------------------------------
  */
@@ -142,7 +145,6 @@ typedef struct {
     int loose;			/* If non-zero, autoscale limits loosely */
     int descending;		/* In non-zero, axis values are descending
 				 * instead of monotonically increasing. */
-    Tk_Anchor anchor;		/* Anchor of tick labels */
     double limits[2];		/* Limits for scaling of axis */
     double prevMin, prevMax;	/* Last limits for scaling of axis */
     double reqStep;		/* Manually selected step size for
@@ -170,17 +172,18 @@ typedef struct {
 
     /* Calculated values */
     unsigned int flags;
-    int baseline;		/* Screen location of baseline */
-    int majorTick;		/* Screen location of major tick */
-    int minorTick;		/* Screen location of minor tick */
-    int tickLabel;		/* Screen location of tick label */
-    int titleX, titleY;		/* Coordinates of axis title */
+
+    Tk_Anchor anchor;		/* Anchor of tick labels */
+    int posArr[4];		/* Screen location of axis, major tick,
+				 * minor tick, and tick label */
+    XPoint titlePos;		/* Coordinates of axis title */
 
     Tcl_Interp *interp;
     unsigned int width, height;	/* Bounding box of axis */
 
     double tickMin, tickMax;	/* Smallest and largest possible major
 				 * ticks on the plot */
+    int tickLength;		/* Legend of major tick on axis. */
     double min, max;		/* Actual (including padding) axis limits */
     double range;		/* Range of values (max-min) */
     double scale;		/* Scale factor to convert values to
@@ -188,14 +191,6 @@ typedef struct {
     int offset;			/* Offset of plotting region from window
 				 * origin */
 
-    XSegment *segArr;		/* Array of computed tick line
-				 * segments. Also includes the axis
-				 * line */
-    int numSegments;		/* Size of the above segment array */
-
-    Label *labelArr;		/* Array of computed tick labels: See the
-				 * description of the Label structure. */
-    int numLabels;		/* Size of the above label array */
     GC lineGC;			/* Graph context for axis lines and ticks */
     GC textGC;			/* Graphic context for tick labels:
 				 * Must be a private GC (can't use
@@ -207,6 +202,13 @@ typedef struct {
 				 * background color is also reset when
 				 * the background color of the graph
 				 * changes. */
+    int numSegments;		/* Size of the above segment array */
+    XSegment *segArr;		/* Array of computed tick line
+				 * segments. Also includes the axis
+				 * line */
+    int numLabels;		/* Size of the above label array */
+    Label *labelArr;		/* Array of computed tick labels: See the
+				 * description of the Label structure. */
 
 } Axis;
 
@@ -227,6 +229,7 @@ typedef struct {
 #define DEF_AXIS_FG_COLOR	BLACK
 #define DEF_AXIS_FG_MONO	BLACK
 #define DEF_AXIS_FONT	 	"*-Courier-Bold-R-Normal-*-100-*"
+#define DEF_AXIS_TICK_LENGTH	"0.1i"
 #define DEF_AXIS_LINE_WIDTH	"0"
 #define DEF_AXIS_LOG_SCALE 	"0"
 #define DEF_AXIS_LOOSE 		"0"
@@ -289,6 +292,8 @@ static Tk_ConfigSpec xAxisConfigSpecs[] =
     {TK_CONFIG_INT, "-subticks", "xSubticks", "AxisSubticks",
 	DEF_AXIS_X_SUBTICKS_BARCHART, Tk_Offset(Axis, reqSubTicks),
 	BARCHART_MASK},
+    {TK_CONFIG_PIXELS, "-ticklength", "xTickLength", "AxisTickLength",
+	DEF_AXIS_TICK_LENGTH, Tk_Offset(Axis, tickLength), ALL_MASK},
     {TK_CONFIG_STRING, "-title", "xTitle", "AxisTitle",
 	"X", Tk_Offset(Axis, title), ALL_MASK | TK_CONFIG_NULL_OK},
     {TK_CONFIG_END, NULL, NULL, NULL, NULL, 0, 0}
@@ -338,6 +343,8 @@ static Tk_ConfigSpec x2AxisConfigSpecs[] =
     {TK_CONFIG_INT, "-subticks", "x2Subticks", "AxisSubticks",
 	DEF_AXIS_SUBTICKS, Tk_Offset(Axis, reqSubTicks),
 	TK_CONFIG_DONT_SET_DEFAULT | ALL_MASK},
+    {TK_CONFIG_PIXELS, "-ticklength", "x2TickLength", "AxisTickLength",
+	DEF_AXIS_TICK_LENGTH, Tk_Offset(Axis, tickLength), ALL_MASK},
     {TK_CONFIG_STRING, "-title", "x2Title", "AxisTitle",
 	DEF_AXIS_TITLE, Tk_Offset(Axis, title), ALL_MASK | TK_CONFIG_NULL_OK},
     {TK_CONFIG_END, NULL, NULL, NULL, NULL, 0, 0}
@@ -386,6 +393,8 @@ static Tk_ConfigSpec yAxisConfigSpecs[] =
 	TK_CONFIG_DONT_SET_DEFAULT | ALL_MASK},
     {TK_CONFIG_INT, "-subticks", "ySubticks", "AxisSubticks",
 	DEF_AXIS_SUBTICKS, Tk_Offset(Axis, reqSubTicks), ALL_MASK},
+    {TK_CONFIG_PIXELS, "-ticklength", "yTickLength", "AxisTickLength",
+	DEF_AXIS_TICK_LENGTH, Tk_Offset(Axis, tickLength), ALL_MASK},
     {TK_CONFIG_STRING, "-title", "yTitle", "AxisTitle",
 	"Y", Tk_Offset(Axis, title), ALL_MASK | TK_CONFIG_NULL_OK},
     {TK_CONFIG_END, NULL, NULL, NULL, NULL, 0, 0}
@@ -434,6 +443,8 @@ static Tk_ConfigSpec y2AxisConfigSpecs[] =
 	TK_CONFIG_DONT_SET_DEFAULT | ALL_MASK},
     {TK_CONFIG_INT, "-subticks", "y2Subticks", "AxisSubticks",
 	DEF_AXIS_SUBTICKS, Tk_Offset(Axis, reqSubTicks), ALL_MASK},
+    {TK_CONFIG_PIXELS, "-ticklength", "y2TickLength", "AxisTickLength",
+	DEF_AXIS_TICK_LENGTH, Tk_Offset(Axis, tickLength), ALL_MASK},
     {TK_CONFIG_STRING, "-title", "y2Title", "AxisTitle",
 	DEF_AXIS_TITLE, Tk_Offset(Axis, title), ALL_MASK | TK_CONFIG_NULL_OK},
     {TK_CONFIG_END, NULL, NULL, NULL, NULL, 0, 0}
@@ -445,10 +456,6 @@ static Tk_ConfigSpec *axisConfigSpecs[4] =
     x2AxisConfigSpecs, y2AxisConfigSpecs	/* Alternate x-axis and y-axis */
 };
 
-#ifndef sun
-extern char *strdup _ANSI_ARGS_((CONST char *s));
-#endif
-
 /* ----------------------------------------------------------------------
  * Custom option parse and print procedures
  * ----------------------------------------------------------------------
@@ -541,27 +548,27 @@ PrintAxisLimit(clientData, tkwin, widgRec, offset, freeProcPtr)
 /*
  * ----------------------------------------------------------------------
  *
- * CreateLabel --
+ * MakeLabel --
  *
  *	Converts a floating point tick value to a string representation.
  *
  * Results:
- *	Returns a malloc'ed copy of the formatted string.
+ *	Returns a formatted label in the string buffer.
  *
  * Side Effects:
  *	Formatted tick label will be displayed on the graph.
  *
  * ----------------------------------------------------------------------
  */
-static char *
-CreateLabel(graphPtr, axisPtr, value)
+static int
+MakeLabel(graphPtr, axisPtr, value, string)
     Graph *graphPtr;		/* Graph widget */
     Axis *axisPtr;		/* Axis structure */
     double value;		/* */
+    char string[];		/* string (length is always
+				 * TCL_DOUBLE_SPACE+1) containing the
+				 * formatted label */
 {
-    char string[TCL_DOUBLE_SPACE + 1];
-    char *result;
-
     if (axisPtr->logScale) {
 	sprintf(string, "1E%d", BLT_RND(value));
     } else {
@@ -578,43 +585,17 @@ CreateLabel(graphPtr, axisPtr, value)
 		string, (char *)NULL) != TCL_OK) {
 	    Tk_BackgroundError(axisPtr->interp);
 	} else {
+	    char *result;
+
 	    result = axisPtr->interp->result;
 	    if (*result != '\0') {
-		result = strdup(result);
+		strncpy(string, result, TCL_DOUBLE_SPACE);
+		string[TCL_DOUBLE_SPACE] = 0;
 		Tcl_ResetResult(axisPtr->interp);
-		return (result);
 	    }
 	}
     }
-    result = strdup(string);
-    return (result);
-}
-
-/*
- * ----------------------------------------------------------------------
- *
- * FreeLabels --
- *
- *	Frees all the text strings allocated in the array of tick labels.
- *
- * Results:
- *	None.
- *
- * Side Effects:
- *	Tick label strings are deallocated.
- *
- * ----------------------------------------------------------------------
- */
-static void
-FreeLabels(labelArr)
-    Label labelArr[];
-{
-    register Label *labelPtr;
-
-    for (labelPtr = labelArr; labelPtr->text != NULL; labelPtr++) {
-	free((char *)labelPtr->text);
-    }
-    free((char *)labelArr);
+    return TCL_OK;
 }
 
 /* Map graph coordinate to normalized coordinates (consider log scale) */
@@ -923,10 +904,10 @@ UpdateLimits(graphPtr, axisPtr)
     /* Indicate if the axis limits have changed */
     result = (axisPtr->limits[LMAX] != axisPtr->prevMax) ||
 	(axisPtr->limits[LMIN] != axisPtr->prevMin);
+    /* and save the previous minimum and maximum values */
+    axisPtr->prevMin = axisPtr->limits[LMIN];
+    axisPtr->prevMax = axisPtr->limits[LMAX];
     if (result) {
-	/* Save the previous minimum and maximum values */
-	axisPtr->prevMin = axisPtr->limits[LMIN];
-	axisPtr->prevMax = axisPtr->limits[LMAX];
 	axisPtr->flags |= AXIS_CONFIG_DIRTY;
     }
 }
@@ -1298,8 +1279,7 @@ SetAxisLimits(graphPtr, axisPtr)
 	axisPtr->flags &= ~AXIS_CONFIG_DIRTY;
 
 	/* When any axis changes, we need to layout the entire graph. */
-	graphPtr->flags |=
-	    (LAYOUT_NEEDED | LAYOUT_ALL | DIRTY | REFRESH);
+	graphPtr->flags |= (LAYOUT_ALL | DIRTY | REFRESH);
     }
 }
 
@@ -1317,14 +1297,11 @@ void
 Blt_ComputeAxes(graphPtr)
     Graph *graphPtr;
 {
-    Axis *axisPtr;
     register int i;
 
     for (i = 0; i < 4; i++) {
-	axisPtr = (Axis *)graphPtr->axisArr[i];
-	SetAxisLimits(graphPtr, axisPtr);
+	SetAxisLimits(graphPtr, (Axis *)graphPtr->axisArr[i]);
     }
-    graphPtr->flags |= LAYOUT_NEEDED;
 }
 
 /*
@@ -1436,6 +1413,7 @@ ConfigureAxis(graphPtr, axisPtr, argc, argv, flags)
      * axis configuration option changes the size of the plotting area
      * (except for -foreground).
      */
+    graphPtr->flags |= LAYOUT_ALL;
     axisPtr->flags |= AXIS_CONFIG_DIRTY;
     SetAxisLimits(graphPtr, axisPtr);
     Blt_RedrawGraph(graphPtr);
@@ -1473,7 +1451,7 @@ DestroyAxis(graphPtr, axis)
 	XFreeGC(graphPtr->display, axisPtr->textGC);
     }
     if (axisPtr->labelArr != NULL) {
-	FreeLabels(axisPtr->labelArr);
+	free((char *)axisPtr->labelArr);
     }
     if (axisPtr->segArr != NULL) {
 	free((char *)axisPtr->segArr);
@@ -1486,7 +1464,7 @@ DestroyAxis(graphPtr, axis)
  *
  * CalculateOffsets --
  *
- *	Determines the placements of the baseline, major and minor ticks,
+ *	Determines the placements of the axis, major and minor ticks,
  *	and title of the axis.
  *
  * Results:
@@ -1502,87 +1480,82 @@ CalculateOffsets(graphPtr, axisPtr)
     int pad;			/* Offset of axis from interior region. This
 				 * includes a possible border and the axis
 				 * line width. */
-    int halfLW;
-    int textHeight;
-    int titleOffset;
-    Axis *oppPtr;		/* Opposite axis */
-    int x2Offset;
+    unsigned int textHeight;
+    int innerPos, outerPos;
+    int majorOffset, minorOffset, labelOffset, titleOffset;
+
+    textHeight = TEXTHEIGHT(graphPtr->fontPtr);
+
+    titleOffset = graphPtr->borderWidth + textHeight;
+    majorOffset = BLT_ABS(axisPtr->tickLength);
+    minorOffset = BLT_RND(majorOffset * 0.5);
+    labelOffset = BLT_RND(majorOffset * 1.4) + axisPtr->lineWidth / 2;
 
     /* Adjust offset for the interior border width and the line width */
-    halfLW = axisPtr->lineWidth / 2;
-    pad = graphPtr->intBW + halfLW + 1;
-    if (graphPtr->intBW > 0) {
+    pad = graphPtr->plotBW + axisPtr->lineWidth + 2;
+    if (graphPtr->plotBW > 0) {
 	pad++;
     }
-    textHeight = TEXTHEIGHT(graphPtr->fontPtr);
-    titleOffset = graphPtr->extBW + textHeight;
-
+    if ((axisPtr->location == LEFT_AXIS) || (axisPtr->location == TOP_AXIS)) {
+	majorOffset = -majorOffset;
+	minorOffset = -minorOffset;
+	labelOffset = -labelOffset;
+    }
     /*
-     * Pre-calculate the x-coordinate positions of the baseline,
+     * Pre-calculate the x-coordinate positions of the axis,
      * tick labels, and the individual major and minor ticks.
      */
-    if (HORIZONTAL_AXIS(axisPtr)) {
-	oppPtr = (Axis *)graphPtr->leftAxis;
-    } else {
-	oppPtr = (Axis *)graphPtr->bottomAxis;
-    }
-
     switch (axisPtr->location) {
     case BOTTOM_AXIS:
-	axisPtr->majorTick = MAPY(oppPtr, -MAJOR_TICK) + pad + halfLW;
-	axisPtr->minorTick = MAPY(oppPtr, -MINOR_TICK) + pad + halfLW;
-	axisPtr->baseline = graphPtr->origin.y + pad;
-	axisPtr->tickLabel = MAPY(oppPtr, -LABEL_TICK) +
-	    (axisPtr->lineWidth + graphPtr->intBW + pad);
-	axisPtr->titleX = (graphPtr->extreme.x + graphPtr->origin.x) / 2;
-	axisPtr->titleY = graphPtr->height - titleOffset;
+	innerPos = graphPtr->origin.y + pad;
+	axisPtr->titlePos.x = (graphPtr->extreme.x + graphPtr->origin.x) / 2;
+	axisPtr->titlePos.y = graphPtr->height - titleOffset;
 	axisPtr->anchor = TK_ANCHOR_N;
 	break;
 
     case LEFT_AXIS:
-	axisPtr->majorTick = MAPX(oppPtr, -MAJOR_TICK) - (pad + halfLW);
-	axisPtr->minorTick = MAPX(oppPtr, -MINOR_TICK) - (pad + halfLW);
-	axisPtr->baseline = graphPtr->origin.x - pad;
-	axisPtr->tickLabel = MAPX(oppPtr, -LABEL_TICK) -
-	    (axisPtr->lineWidth + graphPtr->intBW + pad);
-	axisPtr->titleX = titleOffset;
-	axisPtr->titleY = (graphPtr->origin.y + graphPtr->extreme.y) / 2;
+	innerPos = graphPtr->origin.x - pad;
+	axisPtr->titlePos.x = titleOffset;
+	axisPtr->titlePos.y = (graphPtr->origin.y + graphPtr->extreme.y) / 2;
 	axisPtr->anchor = TK_ANCHOR_E;
 	break;
 
     case TOP_AXIS:
-	axisPtr->majorTick =
-	    MAPY(oppPtr, 1.0 + MAJOR_TICK) - (pad + halfLW);
-	axisPtr->minorTick =
-	    MAPY(oppPtr, 1.0 + MINOR_TICK) - (pad + halfLW);
-	axisPtr->baseline = graphPtr->extreme.y - pad;
-	axisPtr->tickLabel = MAPY(oppPtr, 1.0 + LABEL_TICK) -
-	    (axisPtr->lineWidth + graphPtr->intBW + pad);
-	axisPtr->titleX = (graphPtr->extreme.x + graphPtr->origin.x) / 2;
-	x2Offset = (graphPtr->title != NULL) ? 2 * textHeight : textHeight / 2;
-	axisPtr->titleY = titleOffset + x2Offset;
+	innerPos = graphPtr->extreme.y - pad;
+	axisPtr->titlePos.x = (graphPtr->extreme.x + graphPtr->origin.x) / 2;
+	axisPtr->titlePos.y = titleOffset;
+	if (graphPtr->title != NULL) {
+	    axisPtr->titlePos.y += (2 * textHeight);
+	} else {
+	    axisPtr->titlePos.y += (textHeight / 2);
+	}
 	axisPtr->anchor = TK_ANCHOR_S;
 	break;
 
     case RIGHT_AXIS:
-	axisPtr->majorTick = MAPX(oppPtr, 1.0 + MAJOR_TICK) + pad + halfLW;
-	axisPtr->minorTick = MAPX(oppPtr, 1.0 + MINOR_TICK) + pad + halfLW;
-	axisPtr->baseline = graphPtr->extreme.x + pad;
-	axisPtr->tickLabel = MAPX(oppPtr, 1.0 + LABEL_TICK) +
-	    (axisPtr->lineWidth + graphPtr->intBW + pad);
-	axisPtr->titleX = graphPtr->width -
+	innerPos = graphPtr->extreme.x + pad;
+	axisPtr->titlePos.x = graphPtr->width -
 	    (graphPtr->legendPtr->width + titleOffset);
-	axisPtr->titleY = (graphPtr->origin.y + graphPtr->extreme.y) / 2;
+	axisPtr->titlePos.y = (graphPtr->origin.y + graphPtr->extreme.y) / 2;
 	axisPtr->anchor = TK_ANCHOR_W;
 	break;
 
     }
+    outerPos = innerPos + majorOffset;
+    axisPtr->posArr[MAJOR_TICK] = outerPos;
+    axisPtr->posArr[AXIS_LINE] = innerPos;
+    axisPtr->posArr[MINOR_TICK] = innerPos + minorOffset;
+    axisPtr->posArr[TICK_LABEL] = innerPos + labelOffset;
+    if (axisPtr->tickLength < 0) {
+	axisPtr->posArr[MAJOR_TICK] = innerPos;
+	axisPtr->posArr[AXIS_LINE] = outerPos;
+    }
 }
 
 static XSegment
-Baseline(axisPtr, min, max)
+AxisLine(axisPtr, min, max)
     Axis *axisPtr;		/* Axis information */
-    double min, max;		/* Limits of baseline in graph coordinates */
+    double min, max;		/* Limits of axis in graph coordinates */
 {
     double normMin, normMax;
     XSegment segment;
@@ -1596,11 +1569,11 @@ Baseline(axisPtr, min, max)
 	normMin = 1.0 - normMin;
     }
     if (HORIZONTAL_AXIS(axisPtr)) {
-	segment.y1 = segment.y2 = axisPtr->baseline;
+	segment.y1 = segment.y2 = axisPtr->posArr[AXIS_LINE];
 	segment.x1 = MAPX(axisPtr, normMin);
 	segment.x2 = MAPX(axisPtr, normMax);
     } else {
-	segment.x1 = segment.x2 = axisPtr->baseline;
+	segment.x1 = segment.x2 = axisPtr->posArr[AXIS_LINE];
 	segment.y1 = MAPY(axisPtr, normMin);
 	segment.y2 = MAPY(axisPtr, normMax);
     }
@@ -1609,10 +1582,10 @@ Baseline(axisPtr, min, max)
 
 
 static XSegment
-Tick(axisPtr, value, flag)
+Tick(axisPtr, value, whichTick)
     Axis *axisPtr;
     double value;
-    int flag;			/* If non-zero, create minor tick instead */
+    int whichTick;		/* If non-zero, create minor tick instead */
 {
     double norm;
     XSegment segment;
@@ -1622,13 +1595,13 @@ Tick(axisPtr, value, flag)
     if (axisPtr->descending) {
 	norm = 1.0 - norm;
     }
-    tick = (flag) ? axisPtr->minorTick : axisPtr->majorTick;
+    tick = axisPtr->posArr[whichTick];
     if (HORIZONTAL_AXIS(axisPtr)) {
-	segment.y1 = axisPtr->baseline;
+	segment.y1 = axisPtr->posArr[AXIS_LINE];
 	segment.y2 = tick;
 	segment.x1 = segment.x2 = MAPX(axisPtr, norm);
     } else {
-	segment.x1 = axisPtr->baseline;
+	segment.x1 = axisPtr->posArr[AXIS_LINE];
 	segment.x2 = tick;
 	segment.y1 = segment.y2 = MAPY(axisPtr, norm);
     }
@@ -1640,12 +1613,12 @@ Tick(axisPtr, value, flag)
  *
  * LayoutAxis --
  *
- *	Pre-calculate the x-coordinate positions of the axis baseline,
- *	ticks and labels to be used later when displaying the X axis.
- *	Ticks (minor and major) will be saved in an array of XSegments
- *	so that they can be drawn in one XDrawSegments call. The
- *	strings representing the tick labels and the corresponding
- *	window positions are saved in an array of Label's.
+ *	Pre-calculate the x-coordinate positions of the axis, ticks
+ *	and labels to be used later when displaying the X axis.  Ticks
+ *	(minor and major) will be saved in an array of XSegments so
+ *	that they can be drawn in one XDrawSegments call. The strings
+ *	representing the tick labels and the corresponding window
+ *	positions are saved in an array of Label's.
  *
  *      Calculates the values for each major and minor tick and checks to
  *	see if they are in range (the outer ticks may be outside of the
@@ -1657,8 +1630,7 @@ Tick(axisPtr, value, flag)
  * SideEffects:
  *	Line segments and tick labels saved will be used to draw
  *	the X axis.
- * -----------------------------------------------------------------
- */
+ * ----------------------------------------------------------------- */
 static void
 LayoutAxis(graphPtr, axis)
     Graph *graphPtr;
@@ -1666,7 +1638,7 @@ LayoutAxis(graphPtr, axis)
 {
     Axis *axisPtr = (Axis *)axis;
     XSegment *segArr;
-    unsigned int need;
+    unsigned int arraySize;
     double min, max;
     double value, subValue;
     register int i, j;
@@ -1681,8 +1653,8 @@ LayoutAxis(graphPtr, axis)
 
     /* Save all line coordinates in an array of line segments. */
 
-    need = (1 + (axisPtr->numTicks * (axisPtr->subTicks + 1)));
-    segArr = (XSegment *)malloc(need * sizeof(XSegment));
+    arraySize = (1 + (axisPtr->numTicks * (axisPtr->subTicks + 1)));
+    segArr = (XSegment *)malloc(arraySize * sizeof(XSegment));
     if (segArr == NULL) {
 	return;			/* Can't allocate array of segments */
     }
@@ -1695,14 +1667,14 @@ LayoutAxis(graphPtr, axis)
     }
 
     /* Axis baseline */
-    segArr[0] = Baseline(axisPtr, min, max);
+    segArr[0] = AxisLine(axisPtr, min, max);
 
     sgmts = 1, labels = 0;
     if (!axisPtr->showTicks) {
 	goto done;		/* Only display axis line */
     }
     /* Use numbers just beyond the limits when testing for equality */
-    epsilon = (axisPtr->subStep * 0.1);
+    epsilon = 2 * MIN_DBL_VALUE;
     min -= epsilon, max += epsilon;
 
     value = axisPtr->tickMin;	/* Start from smallest axis tick */
@@ -1717,21 +1689,25 @@ LayoutAxis(graphPtr, axis)
 		subValue += axisPtr->subStep;
 	    }
 	    if ((subValue >= min) && (subValue <= max)) {
-		segArr[sgmts] = Tick(axisPtr, subValue, 1);
+		segArr[sgmts] = Tick(axisPtr, subValue, MINOR_TICK);
 		sgmts++;
 	    }
 	}
 
 	/* Major tick and label */
 	if ((value >= min) && (value <= max)) {
-	    segArr[sgmts] = Tick(axisPtr, value, 0);
+	    short int labelPos;
+
+	    segArr[sgmts] = Tick(axisPtr, value, MAJOR_TICK);
+	    labelPos = (short int)axisPtr->posArr[TICK_LABEL];
 
 	    /* Save tick label position */
+
 	    if (HORIZONTAL_AXIS(axisPtr)) {
 		axisPtr->labelArr[labels].x = segArr[sgmts].x1;
-		axisPtr->labelArr[labels].y = (short int)axisPtr->tickLabel;
+		axisPtr->labelArr[labels].y = labelPos;
 	    } else {
-		axisPtr->labelArr[labels].x = (short int)axisPtr->tickLabel;
+		axisPtr->labelArr[labels].x = labelPos;
 		axisPtr->labelArr[labels].y = segArr[sgmts].y1;
 	    }
 	    sgmts++, labels++;
@@ -1740,19 +1716,13 @@ LayoutAxis(graphPtr, axis)
     }
 
   done:
-#ifdef notdef
-    if (sgmts > need) {
-	fprintf(stderr, "# segments allocated = %d, used = %d\n", need, sgmts);
-    }
-    if (labels > axisPtr->numLabels) {
-	fprintf(stderr, "# labels allocated = %d, used = %d\n",
-	    axisPtr->numLabels, labels);
-    }
-#endif
+
+    assert(sgmts <= arraySize);
+    assert(labels <= axisPtr->numLabels);
+
     if (axisPtr->segArr != NULL) {
 	free((char *)axisPtr->segArr);
     }
-    axisPtr->numLabels = labels;
     axisPtr->segArr = segArr;
     axisPtr->numSegments = sgmts;
 }
@@ -1792,9 +1762,9 @@ DisplayAxis(graphPtr, axis, attrPtr)
     if (axisPtr->title != NULL) {
 	attrPtr->theta = (double)titleRot[axisPtr->location];
 	Blt_DrawText(graphPtr->display, graphPtr->canvas, axisPtr->title,
-	    attrPtr, axisPtr->titleX, axisPtr->titleY);
+	    attrPtr, axisPtr->titlePos.x, axisPtr->titlePos.y);
     }
-    if (axisPtr->numTicks > 0) {
+    if (axisPtr->showTicks) {
 	register int i;
 	TextAttributes textAttr;
 
@@ -1813,10 +1783,10 @@ DisplayAxis(graphPtr, axis, attrPtr)
 		axisPtr->labelArr[i].text, &textAttr,
 		axisPtr->labelArr[i].x, axisPtr->labelArr[i].y);
 	}
-	if (axisPtr->numSegments > 0) {
-	    XDrawSegments(graphPtr->display, graphPtr->canvas,
-		axisPtr->lineGC, axisPtr->segArr, axisPtr->numSegments);
-	}
+    }
+    if (axisPtr->numSegments > 0) {
+	XDrawSegments(graphPtr->display, graphPtr->canvas, axisPtr->lineGC,
+	    axisPtr->segArr, axisPtr->numSegments);
     }
 }
 
@@ -1850,9 +1820,9 @@ PrintAxis(graphPtr, axis, attrPtr)
     if (axisPtr->title != NULL) {
 	attrPtr->theta = (double)titleRot[axisPtr->location];
 	Blt_TextToPostScript(graphPtr, axisPtr->title, attrPtr,
-	    axisPtr->titleX, axisPtr->titleY);
+	    axisPtr->titlePos.x, axisPtr->titlePos.y);
     }
-    if (axisPtr->numTicks > 0) {
+    if (axisPtr->showTicks) {
 	TextAttributes textAttr;
 	register int i;
 
@@ -1868,12 +1838,12 @@ PrintAxis(graphPtr, axis, attrPtr)
 	    Blt_TextToPostScript(graphPtr, axisPtr->labelArr[i].text,
 		&textAttr, axisPtr->labelArr[i].x, axisPtr->labelArr[i].y);
 	}
-	if (axisPtr->numSegments > 0) {
-	    Blt_SetLineAttributes(graphPtr, axisPtr->fgColorPtr,
-		axisPtr->lineWidth, 0);
-	    Blt_SegmentsToPostScript(graphPtr, axisPtr->segArr,
-		axisPtr->numSegments);
-	}
+    }
+    if (axisPtr->numSegments > 0) {
+	Blt_SetLineAttributes(graphPtr, axisPtr->fgColorPtr,
+	    axisPtr->lineWidth, 0);
+	Blt_SegmentsToPostScript(graphPtr, axisPtr->segArr,
+	    axisPtr->numSegments);
     }
 }
 
@@ -1884,13 +1854,17 @@ GetAxisGeometry(graphPtr, axisPtr)
 {
     register int i;
     register int count;
-    char *string;
+    char label[TCL_DOUBLE_SPACE];
+    unsigned int length;
+    unsigned int arraySize, poolSize, used;
+    char *pool;
     unsigned int textWidth, textHeight;
-    unsigned int boxWidth, boxHeight;
+    unsigned int bbWidth, bbHeight;
     int maxWidth, maxHeight;
     double value;
-    double minAxis, maxAxis;
+    double epsilon, minAxis, maxAxis;
     Label *labelArr;
+    int pad;
 
     if ((axisPtr->logScale) || (axisPtr->loose) ||
 	(axisPtr->limits[LMIN] == axisPtr->limits[LMAX])) {
@@ -1902,54 +1876,94 @@ GetAxisGeometry(graphPtr, axisPtr)
 
     /* Use numbers just beyond the limits when testing for equality */
 
-    minAxis -= (axisPtr->subStep * 0.001);
-    maxAxis += (axisPtr->subStep * 0.001);
+    epsilon = 2 * MIN_DBL_VALUE;
+    minAxis -= epsilon, maxAxis += epsilon;
 
-    /* Add an extra slot as a sentinel for the end of the array */
-    labelArr = (Label *)malloc((axisPtr->numTicks + 1) * sizeof(Label));
+    /* Create an array of labels with an attached of characters strings */
+    arraySize = axisPtr->numTicks * sizeof(Label);
+    poolSize = axisPtr->numTicks * AVG_TICK_NUM_CHARS * sizeof(char);
+    labelArr = (Label *)malloc(arraySize + poolSize);
+    pool = (char *)labelArr + arraySize;
 
     textHeight = TEXTHEIGHT(axisPtr->fontPtr);
 
     maxHeight = maxWidth = 0;
-    count = 0;
+    used = count = 0;
     value = axisPtr->tickMin;
     for (i = 0; i < axisPtr->numTicks; i++, value += axisPtr->step) {
 	value = UROUND(value, axisPtr->step);
 	if ((value < minAxis) || (value > maxAxis)) {
-	    continue;		/* out of range */
+	    continue;		/* Out of range */
 	}
-	string = CreateLabel(graphPtr, axisPtr, value);
-	textWidth = Blt_TextStringWidth(axisPtr->fontPtr, string);
-	labelArr[count++].text = string;
+	MakeLabel(graphPtr, axisPtr, value, label);
+	length = strlen(label);
+
+	/*  Resize the label array if we overflow its string pool */
+	if (poolSize <= (used + length)) {
+	    int newSize;
+	    Label *newArr;
+
+	    newSize = poolSize + poolSize;
+	    while (newSize <= (used + length)) {
+		newSize += newSize;
+	    }
+	    newArr = (Label *)malloc(arraySize + newSize);
+	    memcpy((char *)newArr, (char *)labelArr, arraySize + poolSize);
+	    pool = (char *)newArr + arraySize;
+	    free((char *)labelArr);
+	    poolSize = newSize;
+	    labelArr = newArr;
+	}
+	textWidth = Blt_TextStringWidth(axisPtr->fontPtr, label);
+	labelArr[count].text = (pool + used);
+	strcpy(labelArr[count].text, label);
+	used += (length + 1);
+	count++;
 
 	if (axisPtr->theta == 0.0) {
-	    boxWidth = textWidth, boxHeight = textHeight;
+	    bbWidth = textWidth, bbHeight = textHeight;
 	} else {
 	    Blt_GetBoundingBox(textWidth, textHeight, axisPtr->theta,
-		&boxWidth, &boxHeight, (XPoint *)NULL);
+		&bbWidth, &bbHeight, (XPoint *)NULL);
 	}
-	if (boxWidth > maxWidth) {
-	    maxWidth = boxWidth;
+	if (bbWidth > maxWidth) {
+	    maxWidth = bbWidth;
 	}
-	if (boxHeight > maxHeight) {
-	    maxHeight = boxHeight;
+	if (bbHeight > maxHeight) {
+	    maxHeight = bbHeight;
 	}
     }
-    labelArr[count].text = NULL;/* Mark off last string */
-
     if (axisPtr->labelArr != NULL) {
-	FreeLabels(axisPtr->labelArr);
+	free((char *)axisPtr->labelArr);
     }
     axisPtr->labelArr = labelArr;
     axisPtr->numLabels = count;
-    axisPtr->width = maxWidth + axisPtr->lineWidth;
-    axisPtr->height = maxHeight + axisPtr->lineWidth;
+    assert(axisPtr->numLabels <= axisPtr->numTicks);
+
+    /*
+     * Because the axis cap style is "CapProjecting", there's an extra
+     * 1.5 linewidth to be accounted for.
+     */
+    pad = ((axisPtr->lineWidth * 15) / 10) + 2;
+    axisPtr->width = maxWidth + pad;
+    axisPtr->height = maxHeight + pad;
+
+    value = BLT_ABS(axisPtr->tickLength) * 1.4;
+    pad = BLT_RND(value) + graphPtr->plotBW + 1;
+    if (graphPtr->plotBW > 0) {
+	pad++;
+    }
+    if (HORIZONTAL_AXIS(axisPtr)) {
+	axisPtr->height += pad;
+    } else {
+	axisPtr->width += pad;
+    }
 }
 
 void
 Blt_UpdateAxisBackgrounds(graphPtr, colorPtr)
     Graph *graphPtr;
-    XColor *colorPtr;
+    XColor *colorPtr;		/* Background color of graph margin area */
 {
     Axis *axisPtr;
     register int i;
@@ -2053,7 +2067,6 @@ Blt_ComputeLayout(graphPtr)
     Axis *x1, *x2, *y1, *y2;
     unsigned int twiceHeight = (2 * lineHeight);
     unsigned int halfHeight = (lineHeight / 2);
-    double normYScale, normXScale;
     double value;
 
     x1 = (Axis *)graphPtr->bottomAxis;
@@ -2065,19 +2078,18 @@ Blt_ComputeLayout(graphPtr)
     left = ((y1->mapped) && (y1->title != NULL)) ? twiceHeight : halfHeight;
     bottom = ((x1->mapped) && (x1->title != NULL)) ? twiceHeight : halfHeight;
     right = ((y2->mapped) && (y2->title != NULL)) ? twiceHeight : 0;
+
     if ((x2->mapped) && (x2->title != NULL)) {
 	top += twiceHeight;
     }
     GetAxisGeometry(graphPtr, x1);
     maxTickWidth = 0;
-    normXScale = normYScale = 1.0 + LABEL_TICK;
     if ((x1->mapped) && (x1->showTicks)) {
 	bottom += x1->height;
     }
     GetAxisGeometry(graphPtr, x2);
     if ((x2->mapped) && (x2->showTicks)) {
 	top += x2->height;
-	normYScale += LABEL_TICK;
     }
     GetAxisGeometry(graphPtr, y1);
     if ((y1->mapped) && (y1->showTicks)) {
@@ -2086,7 +2098,6 @@ Blt_ComputeLayout(graphPtr)
     GetAxisGeometry(graphPtr, y2);
     if ((y2->mapped) && (y2->showTicks)) {
 	right += y2->width + PADX;
-	normXScale += LABEL_TICK;
     }
     /* Override calculated values if user specified margins */
 
@@ -2099,7 +2110,7 @@ Blt_ComputeLayout(graphPtr)
     if (graphPtr->bottomMargin > 0) {
 	bottom = graphPtr->bottomMargin;
     }
-    borderWidths = graphPtr->extBW + graphPtr->intBW;
+    borderWidths = graphPtr->borderWidth + graphPtr->plotBW;
     height = graphPtr->height - ((2 * borderWidths) + top + bottom);
     (*graphPtr->legendPtr->geomProc) (graphPtr, height);
     if ((graphPtr->legendPtr->mapped) && (graphPtr->legendPtr->useDefault)) {
@@ -2127,20 +2138,12 @@ Blt_ComputeLayout(graphPtr)
     if (leftOver < 0) {
 	return TCL_ERROR;
     }
-    x1->scale = leftOver / normXScale;	/* Pixels per X unit */
+    x1->scale = (double)leftOver;	/* Pixels per X unit */
     leftOver = graphPtr->height - (top + bottom);
     if (leftOver < 0) {
 	return TCL_ERROR;
     }
-    y1->scale = leftOver / normYScale;	/* Pixels per Y unit */
-
-    /* Add tick distance to center graph */
-
-    value = LABEL_TICK * x1->scale;
-    x1->offset += BLT_RND(value);
-    value = LABEL_TICK * y1->scale;
-    y1->offset -= BLT_RND(value);
-
+    y1->scale = (double)leftOver;	/* Pixels per Y unit */
     x2->scale = x1->scale;
     x2->offset = x1->offset;
     y2->scale = y1->scale;
@@ -2180,6 +2183,7 @@ GetAxisLimits(axisPtr, argc, argv)
 
 {
     char string[TCL_DOUBLE_SPACE + 1];
+    double min, max;
 
     if (argc != 3) {
 	Tcl_AppendResult(axisPtr->interp, "wrong # args: should be \"",
@@ -2187,9 +2191,14 @@ GetAxisLimits(axisPtr, argc, argv)
 	    (char *)NULL);
 	return TCL_ERROR;
     }
-    Tcl_PrintDouble(axisPtr->interp, axisPtr->min, string);
+    min = axisPtr->min, max = axisPtr->max;
+    if (axisPtr->logScale) {
+	min = BLT_EXP10(min);
+	max = BLT_EXP10(max);
+    }
+    Tcl_PrintDouble(axisPtr->interp, min, string);
     Tcl_AppendElement(axisPtr->interp, string);
-    Tcl_PrintDouble(axisPtr->interp, axisPtr->max, string);
+    Tcl_PrintDouble(axisPtr->interp, max, string);
     Tcl_AppendElement(axisPtr->interp, string);
     return TCL_OK;
 }
@@ -2298,8 +2307,9 @@ Blt_CreateAxis(graphPtr, type, flags)
 	graphPtr->interp->result = "can't allocate axis structure";
 	return TCL_ERROR;
     }
-    location = (AxisLocation) type;	/* For now, there's a 1-1 correspondence
-				    * between axis types and locations */
+    location = (AxisLocation) type;	/* For now, there's a 1-1
+					 * correspondence between axis
+					 * types and locations */
     axisPtr->type = type;
     axisPtr->location = location;
     axisPtr->interp = graphPtr->interp;	/* Needed for Tcl_PrintDouble */
@@ -2309,6 +2319,12 @@ Blt_CreateAxis(graphPtr, type, flags)
     axisPtr->displayProc = DisplayAxis;
     axisPtr->layoutProc = LayoutAxis;
     axisPtr->printProc = PrintAxis;
+
+    /*
+     * The actual axis min and max can't be the same, so initializing
+     * the previous limits to zero is Ok.
+     */
+    axisPtr->prevMin = axisPtr->prevMax = 0.0;
     axisPtr->mapped = ((location == BOTTOM_AXIS) || (location == LEFT_AXIS));
 
     graphPtr->axisArr[type] = (GraphAxis *)axisPtr;
