@@ -330,7 +330,7 @@ PrintVector(clientData, tkwin, widgRec, offset, freeProcPtr)
     register int i;
     char *result;
     Tcl_DString buffer;
-    char string[TCL_DOUBLE_SPACE];
+    char string[TCL_DOUBLE_SPACE + 1];
 
     vecPtr = (flags & ANY_X_MASK) ? &(elemPtr->x) : &(elemPtr->y);
     if (vecPtr->length == 0) {
@@ -433,7 +433,7 @@ PrintCoordPairs(clientData, tkwin, widgRec, offset, freeProcPtr)
     register int i;
     char *result;
     int length;
-    char string[TCL_DOUBLE_SPACE];
+    char string[TCL_DOUBLE_SPACE + 1];
     Tcl_DString buffer;
 
     result = "";
@@ -874,7 +874,7 @@ ResetDisplayList(graphPtr, newList)
 	}
     }
     free((char *)nameArr);
-    graphPtr->flags |= (DIRTY | LAYOUT_NEEDED | LAYOUT_ALL);
+    graphPtr->flags |= (DIRTY | LAYOUT_ALL);
     Blt_ComputeAxes(graphPtr);
     Blt_RedrawGraph(graphPtr);
     return TCL_OK;
