@@ -189,10 +189,10 @@ ConfigureCrosshairs(graphPtr)
     gcValues.function = GXxor;
 
     /* The graph's color option may not have been set yet */
-    if (graphPtr->intBgColorPtr == NULL) {
+    if (graphPtr->plotBg == NULL) {
 	colorValue = WhitePixelOfScreen(Tk_Screen(graphPtr->tkwin));
     } else {
-	colorValue = graphPtr->intBgColorPtr->pixel;
+	colorValue = graphPtr->plotBg->pixel;
     }
     gcValues.background = colorValue;
     gcValues.foreground = (colorValue ^ hairsPtr->colorPtr->pixel);
