@@ -30,9 +30,6 @@
 #include <X11/Xutil.h>
 
 extern int strcasecmp _ANSI_ARGS_((CONST char *s1, CONST char *s2));
-#ifndef sun
-extern char *strdup _ANSI_ARGS_((CONST char *s));
-#endif
 
 static int ParsePosition _ANSI_ARGS_((ClientData, Tcl_Interp *, Tk_Window,
 	char *, char *, int));
