@@ -1,4 +1,3 @@
-
 /*
  * bltGraph.c --
  *
@@ -106,7 +105,7 @@ static Tk_ConfigSpec configSpecs[] =
     {TK_CONFIG_SYNONYM, "-bg", "background", (char *)NULL, (char *)NULL, 0,
 	ALL_MASK},
     {TK_CONFIG_PIXELS, "-borderwidth", "borderWidth", "BorderWidth",
-	DEF_GRAPH_BORDER_WIDTH, Tk_Offset(Graph, extBW),
+	DEF_GRAPH_BORDER_WIDTH, Tk_Offset(Graph, borderWidth),
 	TK_CONFIG_DONT_SET_DEFAULT | ALL_MASK},
     {TK_CONFIG_PIXELS, "-bottommargin", "bottomMargin", "Margin",
 	DEF_GRAPH_MARGIN, Tk_Offset(Graph, bottomMargin), ALL_MASK},
@@ -121,10 +120,10 @@ static Tk_ConfigSpec configSpecs[] =
     {TK_CONFIG_SYNONYM, "-fg", "foreground", (char *)NULL, (char *)NULL, 0,
 	ALL_MASK},
     {TK_CONFIG_COLOR, "-foreground", "foreground", "Foreground",
-	DEF_GRAPH_FG_COLOR, Tk_Offset(Graph, fgColorPtr),
+	DEF_GRAPH_FG_COLOR, Tk_Offset(Graph, marginFg),
 	TK_CONFIG_COLOR_ONLY | ALL_MASK},
     {TK_CONFIG_COLOR, "-foreground", "foreground", "Foreground",
-	DEF_GRAPH_FG_MONO, Tk_Offset(Graph, fgColorPtr),
+	DEF_GRAPH_FG_MONO, Tk_Offset(Graph, marginFg),
 	TK_CONFIG_MONO_ONLY | ALL_MASK},
     {TK_CONFIG_PIXELS, "-halo", "halo", "Halo",
 	DEF_GRAPH_HALO_BAR, Tk_Offset(Graph, halo), BARCHART_MASK},
@@ -140,22 +139,22 @@ static Tk_ConfigSpec configSpecs[] =
 	DEF_GRAPH_MARGIN, Tk_Offset(Graph, leftMargin),
 	TK_CONFIG_DONT_SET_DEFAULT | ALL_MASK},
     {TK_CONFIG_COLOR, "-plotbackground", "plotBackground", "Background",
-	DEF_GRAPH_PLOT_BG_MONO, Tk_Offset(Graph, intBgColorPtr),
+	DEF_GRAPH_PLOT_BG_MONO, Tk_Offset(Graph, plotBg),
 	TK_CONFIG_MONO_ONLY | ALL_MASK},
     {TK_CONFIG_COLOR, "-plotbackground", "plotBackground", "Background",
-	DEF_GRAPH_PLOT_BG_COLOR, Tk_Offset(Graph, intBgColorPtr),
+	DEF_GRAPH_PLOT_BG_COLOR, Tk_Offset(Graph, plotBg),
 	TK_CONFIG_COLOR_ONLY | ALL_MASK},
     {TK_CONFIG_INT, "-plotborderwidth", "plotBorderWidth", "BorderWidth",
-	DEF_GRAPH_PLOT_BW_COLOR, Tk_Offset(Graph, intBW),
+	DEF_GRAPH_PLOT_BW_COLOR, Tk_Offset(Graph, plotBW),
 	ALL_MASK | TK_CONFIG_COLOR_ONLY},
     {TK_CONFIG_INT, "-plotborderwidth", "plotBorderWidth", "BorderWidth",
-	DEF_GRAPH_PLOT_BW_MONO, Tk_Offset(Graph, intBW),
+	DEF_GRAPH_PLOT_BW_MONO, Tk_Offset(Graph, plotBW),
 	ALL_MASK | TK_CONFIG_MONO_ONLY},
     {TK_CONFIG_RELIEF, "-plotrelief", "plotRelief", "Relief",
-	DEF_GRAPH_PLOT_RELIEF, Tk_Offset(Graph, intRelief),
+	DEF_GRAPH_PLOT_RELIEF, Tk_Offset(Graph, plotRelief),
 	TK_CONFIG_DONT_SET_DEFAULT | ALL_MASK},
     {TK_CONFIG_RELIEF, "-relief", "relief", "Relief",
-	DEF_GRAPH_RELIEF, Tk_Offset(Graph, extRelief),
+	DEF_GRAPH_RELIEF, Tk_Offset(Graph, relief),
 	TK_CONFIG_DONT_SET_DEFAULT | ALL_MASK},
     {TK_CONFIG_PIXELS, "-rightmargin", "rightMargin", "Margin",
 	DEF_GRAPH_MARGIN, Tk_Offset(Graph, rightMargin),
@@ -259,7 +258,7 @@ GraphEventProc(clientData, eventPtr)
 	}
 	Tk_EventuallyFree((ClientData)graphPtr, DestroyGraph);
     } else if (eventPtr->type == ConfigureNotify) {
-	graphPtr->flags |= (LAYOUT_NEEDED | LAYOUT_ALL | REFRESH);
+	graphPtr->flags |= (LAYOUT_ALL | REFRESH);
 	Blt_RedrawGraph(graphPtr);
     }
 }
@@ -462,11 +461,11 @@ DestroyGraph(clientData)
     (*graphPtr->crosshairs->destroyProc) (graphPtr);
 
     /* Release allocated X resources and memory. */
-    if (graphPtr->extGC != NULL) {
-	Tk_FreeGC(graphPtr->display, graphPtr->extGC);
+    if (graphPtr->marginGC != NULL) {
+	Tk_FreeGC(graphPtr->display, graphPtr->marginGC);
     }
-    if (graphPtr->extBgGC != NULL) {
-	Tk_FreeGC(graphPtr->display, graphPtr->extBgGC);
+    if (graphPtr->marginFillGC != NULL) {
+	Tk_FreeGC(graphPtr->display, graphPtr->marginFillGC);
     }
     if (graphPtr->elemMap != None) {
 	XFreePixmap(graphPtr->display, graphPtr->elemMap);
@@ -527,8 +526,9 @@ CreateGraph(interp, parent, pathName, type)
     graphPtr->width = graphPtr->height = 400;
     graphPtr->reqWidth = graphPtr->reqHeight = 400;
     graphPtr->buffered = 1;
-    graphPtr->intRelief = TK_RELIEF_SUNKEN;
-    graphPtr->extRelief = TK_RELIEF_FLAT;
+    graphPtr->plotRelief = TK_RELIEF_SUNKEN;
+    graphPtr->relief = TK_RELIEF_FLAT;
+    graphPtr->flags |= (DIRTY | LAYOUT_ALL);
 
     Tcl_InitHashTable(&(graphPtr->elemTable), TCL_STRING_KEYS);
     Blt_InitLinkedList(&(graphPtr->elemList), TCL_STRING_KEYS);
@@ -602,7 +602,7 @@ ConfigureGraph(graphPtr, argc, argv, flags)
     char **argv;		/* Configuration arguments */
     unsigned int flags;		/* Configuration flags */
 {
-    XColor *borderColor;
+    XColor *colorPtr;
     GC newGC;
     XGCValues gcValues;
     unsigned long gcMask;
@@ -619,40 +619,40 @@ ConfigureGraph(graphPtr, argc, argv, flags)
     }
     Tk_GeometryRequest(graphPtr->tkwin, graphPtr->reqWidth,
 	graphPtr->reqHeight);
-    Tk_SetInternalBorder(graphPtr->tkwin, graphPtr->extBW);
+    Tk_SetInternalBorder(graphPtr->tkwin, graphPtr->borderWidth);
     Tk_SetBackgroundFromBorder(graphPtr->tkwin, graphPtr->border);
 
-    borderColor = Tk_3DBorderColor(graphPtr->border);
+    colorPtr = Tk_3DBorderColor(graphPtr->border);
 
     /* Update background color for axis text GCs */
-    Blt_UpdateAxisBackgrounds(graphPtr, borderColor);
+    Blt_UpdateAxisBackgrounds(graphPtr, colorPtr);
 
     /*
      * Create GCs for interior and exterior regions, and a background
      * GC for clearing the margins with XFillRectangle
      */
-    gcValues.foreground = graphPtr->fgColorPtr->pixel;
-    gcValues.background = borderColor->pixel;
+    gcValues.foreground = graphPtr->marginFg->pixel;
+    gcValues.background = colorPtr->pixel;
     gcValues.font = graphPtr->fontPtr->fid;
     gcMask = (GCForeground | GCBackground | GCFont);
     newGC = Tk_GetGC(graphPtr->tkwin, gcMask, &gcValues);
-    if (graphPtr->extGC != NULL) {
-	Tk_FreeGC(graphPtr->display, graphPtr->extGC);
+    if (graphPtr->marginGC != NULL) {
+	Tk_FreeGC(graphPtr->display, graphPtr->marginGC);
     }
-    graphPtr->extGC = newGC;
-    gcValues.foreground = borderColor->pixel;
-    gcValues.background = graphPtr->fgColorPtr->pixel;
+    graphPtr->marginGC = newGC;
+    gcValues.foreground = colorPtr->pixel;
+    gcValues.background = graphPtr->marginFg->pixel;
     newGC = Tk_GetGC(graphPtr->tkwin, gcMask, &gcValues);
-    if (graphPtr->extBgGC != NULL) {
-	Tk_FreeGC(graphPtr->display, graphPtr->extBgGC);
+    if (graphPtr->marginFillGC != NULL) {
+	Tk_FreeGC(graphPtr->display, graphPtr->marginFillGC);
     }
-    graphPtr->extBgGC = newGC;
-    gcValues.foreground = graphPtr->intBgColorPtr->pixel;
+    graphPtr->marginFillGC = newGC;
+    gcValues.foreground = graphPtr->plotBg->pixel;
     newGC = Tk_GetGC(graphPtr->tkwin, gcMask, &gcValues);
-    if (graphPtr->intGC != NULL) {
-	Tk_FreeGC(graphPtr->display, graphPtr->intGC);
+    if (graphPtr->plotFillGC != NULL) {
+	Tk_FreeGC(graphPtr->display, graphPtr->plotFillGC);
     }
-    graphPtr->intGC = newGC;
+    graphPtr->plotFillGC = newGC;
     /*
      * If the -inverted option changed, we need to readjust the pointers
      * to the axes and recompute the their scales.
@@ -689,7 +689,7 @@ ConfigureGraph(graphPtr, argc, argv, flags)
      */
     if (Blt_OptionChanged(configSpecs, "-invertxy", "-title", "-font",
 	    "-*margin", "-*width", "-height", (char *)NULL)) {
-	graphPtr->flags |= (DIRTY | LAYOUT_NEEDED | LAYOUT_ALL);
+	graphPtr->flags |= (DIRTY | LAYOUT_ALL);
     }
     graphPtr->flags |= REFRESH;
     Blt_RedrawGraph(graphPtr);
@@ -902,7 +902,7 @@ Blt_LayoutGraph(graphPtr)
     for (entryPtr = Blt_FirstListEntry(&(graphPtr->elemList));
 	entryPtr != NULL; entryPtr = Blt_NextListEntry(entryPtr)) {
 	elemPtr = (Element *)Blt_GetListValue(entryPtr);
-	if ((graphPtr->flags & LAYOUT_ALL) ||
+	if ((graphPtr->flags & LAYOUT_DIRTY) ||
 	    (elemPtr->flags & LAYOUT_NEEDED)) {
 	    (*elemPtr->layoutProc) (graphPtr, elemPtr);
 	    elemPtr->flags &= ~LAYOUT_NEEDED;
@@ -913,13 +913,13 @@ Blt_LayoutGraph(graphPtr)
     for (entryPtr = Blt_FirstListEntry(&(graphPtr->tagList));
 	entryPtr != NULL; entryPtr = Blt_NextListEntry(entryPtr)) {
 	tagPtr = (Tag *)Blt_GetListValue(entryPtr);
-	if ((graphPtr->flags & LAYOUT_ALL) ||
+	if ((graphPtr->flags & LAYOUT_DIRTY) ||
 	    (tagPtr->flags & LAYOUT_NEEDED)) {
 	    (*tagPtr->layoutProc) (graphPtr, tagPtr);
 	    tagPtr->flags &= ~LAYOUT_NEEDED;
 	}
     }
-    graphPtr->flags &= ~(LAYOUT_ALL | LAYOUT_NEEDED);
+    graphPtr->flags &= ~LAYOUT_ALL;
 }
 
 /*
@@ -1086,16 +1086,16 @@ DisplayExterior(graphPtr, rectPtr)
     rectArr[2].height = rectArr[1].height = rectPtr->height;
     rectArr[2].x = rectPtr->x + rectPtr->width;
     rectArr[2].width = graphPtr->width - rectArr[2].x;
-    XFillRectangles(graphPtr->display, graphPtr->canvas, graphPtr->extBgGC,
-	rectArr, 4);
+    XFillRectangles(graphPtr->display, graphPtr->canvas,
+	graphPtr->marginFillGC, rectArr, 4);
 
     /* Interior 3D border */
 
-    if ((graphPtr->intRelief != TK_RELIEF_FLAT) &&
-	(graphPtr->intBW > 0)) {
+    if ((graphPtr->plotRelief != TK_RELIEF_FLAT) &&
+	(graphPtr->plotBW > 0)) {
 	Tk_Draw3DRectangle(graphPtr->display, graphPtr->canvas,
 	    graphPtr->border, rectPtr->x, rectPtr->y, rectPtr->width,
-	    rectPtr->height, graphPtr->intBW, graphPtr->intRelief);
+	    rectPtr->height, graphPtr->plotBW, graphPtr->plotRelief);
     }
     /* Legend */
     if (graphPtr->legendPtr->useDefault) {
@@ -1105,13 +1105,13 @@ DisplayExterior(graphPtr, rectPtr)
 
     textAttr.theta = 0.0;
     textAttr.anchor = TK_ANCHOR_CENTER;
-    textAttr.gc = graphPtr->extGC;
+    textAttr.gc = graphPtr->marginGC;
     textAttr.bgColorPtr = Tk_3DBorderColor(graphPtr->border);
     textAttr.fontPtr = graphPtr->fontPtr;
 
     if (graphPtr->title != NULL) {
 	x = (graphPtr->extreme.x + graphPtr->origin.x) / 2;
-	y = graphPtr->extBW + TEXTHEIGHT(graphPtr->fontPtr);
+	y = graphPtr->borderWidth + TEXTHEIGHT(graphPtr->fontPtr);
 	Blt_DrawText(graphPtr->display, graphPtr->canvas, graphPtr->title,
 	    &textAttr, x, y);
     }
@@ -1123,11 +1123,11 @@ DisplayExterior(graphPtr, rectPtr)
 	}
     }
     /* Exterior 3D border */
-    if ((graphPtr->extRelief != TK_RELIEF_FLAT) &&
-	(graphPtr->extBW > 0)) {
+    if ((graphPtr->relief != TK_RELIEF_FLAT) &&
+	(graphPtr->borderWidth > 0)) {
 	Tk_Draw3DRectangle(graphPtr->display, graphPtr->canvas,
 	    graphPtr->border, 0, 0, graphPtr->width, graphPtr->height,
-	    graphPtr->extBW, graphPtr->extRelief);
+	    graphPtr->borderWidth, graphPtr->relief);
     }
 }
 
@@ -1172,7 +1172,7 @@ DisplayGraph(clientData)
     graphPtr->height = Tk_Height(graphPtr->tkwin);
 
     if (graphPtr->flags & LAYOUT_NEEDED) {
-	if (graphPtr->flags & LAYOUT_ALL) {
+	if (graphPtr->flags & LAYOUT_DIRTY) {
 	    if (Blt_ComputeLayout(graphPtr) != TCL_OK) {
 		return;		/* Not enough room to plot graph */
 	    }
@@ -1182,9 +1182,9 @@ DisplayGraph(clientData)
     /*
      * Determine the plotting surface region of the graph window
      */
-    twiceBW = (graphPtr->intBW * 2);
-    plotArea.x = graphPtr->origin.x - graphPtr->intBW;
-    plotArea.y = graphPtr->extreme.y - graphPtr->intBW;
+    twiceBW = (graphPtr->plotBW * 2);
+    plotArea.x = graphPtr->origin.x - graphPtr->plotBW;
+    plotArea.y = graphPtr->extreme.y - graphPtr->plotBW;
     plotArea.width = (graphPtr->extreme.x - graphPtr->origin.x) + twiceBW;
     plotArea.height = (graphPtr->origin.y - graphPtr->extreme.y) + twiceBW;
 
@@ -1219,7 +1219,7 @@ DisplayGraph(clientData)
 	     * Clear the background and draw the elements into the pixmap.
 	     */
 	    XFillRectangles(graphPtr->display, graphPtr->elemMap,
-		graphPtr->intGC, &plotArea, 1);
+		graphPtr->plotFillGC, &plotArea, 1);
 	    save = graphPtr->canvas;
 	    graphPtr->canvas = graphPtr->elemMap;
 	    DisplayElements(graphPtr);
@@ -1227,12 +1227,12 @@ DisplayGraph(clientData)
 	    graphPtr->flags &= ~DIRTY;
 	}
 	XCopyArea(graphPtr->display, graphPtr->elemMap, graphPtr->canvas,
-	    graphPtr->extGC, plotArea.x, plotArea.y,
+	    graphPtr->marginGC, plotArea.x, plotArea.y,
 	    (unsigned int)plotArea.width, (unsigned int)plotArea.height,
 	    plotArea.x, plotArea.y);
     } else {
-	XFillRectangles(graphPtr->display, graphPtr->canvas, graphPtr->intGC,
-	    &plotArea, 1);
+	XFillRectangles(graphPtr->display, graphPtr->canvas,
+	    graphPtr->plotFillGC, &plotArea, 1);
 	DisplayElements(graphPtr);
     }
     DisplayTags(graphPtr);
@@ -1250,17 +1250,17 @@ DisplayGraph(clientData)
     if (graphPtr->flags & REFRESH) {
 	DisplayExterior(graphPtr, &plotArea);
 	XCopyArea(graphPtr->display, graphPtr->canvas,
-	    Tk_WindowId(graphPtr->tkwin), graphPtr->extGC, 0, 0,
+	    Tk_WindowId(graphPtr->tkwin), graphPtr->marginGC, 0, 0,
 	    graphPtr->width, graphPtr->height, 0, 0);
     } else {
 	/* Draw just the interior plotting region */
 
-	plotArea.x += graphPtr->intBW;
-	plotArea.y += graphPtr->intBW;
-	plotArea.width -= (graphPtr->intBW * 2);
-	plotArea.height -= (graphPtr->intBW * 2);
+	plotArea.x += graphPtr->plotBW;
+	plotArea.y += graphPtr->plotBW;
+	plotArea.width -= (graphPtr->plotBW * 2);
+	plotArea.height -= (graphPtr->plotBW * 2);
 	XCopyArea(graphPtr->display, graphPtr->canvas,
-	    Tk_WindowId(graphPtr->tkwin), graphPtr->extGC, plotArea.x,
+	    Tk_WindowId(graphPtr->tkwin), graphPtr->marginGC, plotArea.x,
 	    plotArea.y, (unsigned int)plotArea.width,
 	    (unsigned int)plotArea.height, plotArea.x, plotArea.y);
     }
